#include "stdafx.h"
#include "Shader_Water.h"

#ifdef CPU_NORMALS
int gridsize_x = 256, gridsize_y = 512;		
#else
int gridsize_x = 128, gridsize_y = 256;		
#endif

Shader_Water::Shader_Water()
{
	g_pd3dDevice		= NULL;
	g_depthstencil		= NULL;
	terrain_texture		= NULL;
	terrain_effect		= NULL;
}

Shader_Water::~Shader_Water()
{
	if( g_depthstencil != NULL )
	{
		g_depthstencil->Release();
		g_depthstencil = NULL;
	}
	if( terrain_texture != NULL )
	{
		terrain_texture->Release();
		terrain_texture = NULL;
	}
	if( terrain_effect != NULL )
	{
		terrain_effect->Release();
		terrain_effect = NULL;
	}
	
	delete camera_mouse;
	delete sea;
}

void Shader_Water::Init(LPDIRECT3DDEVICE9 pDevice, D3DXVECTOR3 position, float Width, float Height,
				const LPWSTR WaterTextureName, const LPWSTR SkyTextureName, camera_info c_info)
{
	g_pd3dDevice = pDevice;
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW  );		
	g_pd3dDevice->GetDepthStencilSurface(&g_depthstencil);

	// initialize camera objects
	camera_mouse = new camera(c_info.pos,
							c_info.rotx,
							c_info.roty,
							c_info.rotz,
							c_info.fov,
							c_info.aspect,
							c_info.nearz,
							c_info.farz);
//	camera_view = new camera(D3DXVECTOR3(-150,50,-100),-0.5,-1.3,0,45,1.33,0.1,10000.0);
	
	SetParameta();
	
	// create sea object
	sea = new surface(&D3DXVECTOR3(0,0,0), &D3DXVECTOR3(0,1,0), 
		gridsize_x, gridsize_y, g_pd3dDevice,
		camera_mouse, prm);

	// load terrain texture
	if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, WaterTextureName, &terrain_texture ) ) )
	{
		MessageBox(NULL, L"Could not find terrain texture", L"terrian", MB_OK);		
	}
	// load terrain underwater effect

	wchar_t *errortext;
	LPD3DXBUFFER errors;
	D3DXHANDLE hTechnique;

	D3DXCreateEffectFromFile(g_pd3dDevice, L"Data/Effect/terrain_underwater.fx", 
		NULL, NULL, 0, NULL, &terrain_effect, &errors );

	if (errors != NULL){
		errortext = (wchar_t*) errors->GetBufferPointer();
		MessageBox(NULL, errortext, L"terrain_underwater.fx", MB_OK);		
	}

	terrain_effect->FindNextValidTechnique(NULL, &hTechnique);    
	terrain_effect->SetTechnique(hTechnique);
}

void Shader_Water::loadpreset(int n)
{
	switch(n)
	{
	case 1:		// Ϲ ٴ ȿ
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	0.9f );
		prm->set_float( p_fFalloff,		0.607f );
		prm->set_float( p_fWaterColourR,	0.07f );
		prm->set_float( p_fWaterColourG,	0.11f );
		prm->set_float( p_fWaterColourB,	0.11f );
		prm->set_float(	p_fLODbias,		0.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_bool(	p_bDiffuseRefl,		false );
		break;
	case 2:		// ȣ ȿ.   
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	4.0f );
		prm->set_float( p_fFalloff,		0.47f );
		prm->set_float( p_fWaterColourR,	0.13f );
		prm->set_float( p_fWaterColourG,	0.19f );
		prm->set_float( p_fWaterColourB,	0.22f );
		prm->set_float(	p_fLODbias,		4.5f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_bool(	p_bDiffuseRefl,		false );
		break;
	case 3:		//   ٴ
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	7.0f );
		prm->set_float( p_fFalloff,		0.53f );
		prm->set_float( p_fWaterColourR,	0.12f );
		prm->set_float( p_fWaterColourG,	0.22f );
		prm->set_float( p_fWaterColourB,	0.29f );
		prm->set_float(	p_fLODbias,		10.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		false );
		prm->set_bool(	p_bDrawIsland,		false );
		prm->set_bool(	p_bDiffuseRefl,		true );
		prm->set_float( p_fSunPosAlpha,		2.91f );
		prm->set_float( p_fSunPosTheta,		0.36f );
		prm->set_float( p_fSunShininess,	1263.0f );
		prm->set_float( p_fSunStrength,		208.0f );
		break;
	case 4:		// ׷ 
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	4.0f );
		prm->set_float( p_fFalloff,		0.47f );
		prm->set_float( p_fWaterColourR,	0.13f );
		prm->set_float( p_fWaterColourG,	0.19f );
		prm->set_float( p_fWaterColourB,	0.22f );
		prm->set_float(	p_fLODbias,		4.5f);
		prm->set_bool(	p_bAsPoints,		true );
		prm->set_bool(	p_bDrawDuckie,		false );
		prm->set_bool(	p_bDrawIsland,		false );
		prm->set_bool(	p_bDiffuseRefl,		true );
		break;
	case 5:		//   ٴȿ
		prm->set_float( p_fScale,		0.197f );
		prm->set_float( p_fStrength,	12.9f );
		prm->set_float( p_fFalloff,		0.467f );
		prm->set_float( p_fWaterColourR,	0.12f );
		prm->set_float( p_fWaterColourG,	0.20f );
		prm->set_float( p_fWaterColourB,	0.24f );
		prm->set_float(	p_fLODbias,			0.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_float( p_fSunPosAlpha,		2.91f );
		prm->set_float( p_fSunPosTheta,		0.88f );
		prm->set_float( p_fSunShininess,	1263.0f );
		prm->set_float( p_fSunStrength,		5270.0f );
		prm->set_bool(	p_bDiffuseRefl,		true );
		break;
	case 6:		//   ٴȿ
		prm->set_float( p_fScale,		0.38f );
		prm->set_float( p_fStrength,	11.3f );
		prm->set_float( p_fFalloff,		0.56f );
		prm->set_float( p_fWaterColourR,	0.17f );
		prm->set_float( p_fWaterColourG,	0.27f );
		prm->set_float( p_fWaterColourB,	0.26f );
		prm->set_float(	p_fLODbias,			0.0f);
		prm->set_bool(	p_bAsPoints,		false );
		prm->set_bool(	p_bDrawDuckie,		true );
		prm->set_bool(	p_bDrawIsland,		true );
		prm->set_float( p_fSunPosAlpha,		2.91f );
		prm->set_float( p_fSunPosTheta,		0.256f );
		prm->set_float( p_fSunShininess,	700.0f );
		prm->set_float( p_fSunStrength,		200.85f );
		prm->set_bool(	p_bDiffuseRefl,		false );
		break;
	}
}

void Shader_Water::SetParameta()
{
	// Ϲ  ǥ 
	// create parameter handler
	prm = new parameterhandler();
	prm->set_float( p_fStrength,	0.9f );
	prm->set_bool(	p_bDisplace,	true );
	prm->set_int(	p_iOctaves,		8 );
	prm->set_float( p_fScale,		0.38f );
	prm->set_float( p_fFalloff,		0.607f );
	prm->set_float( p_fAnimspeed,	1.4f );
	prm->set_float( p_fTimemulti,	1.27f );
	prm->set_bool(	p_bPaused,		false );

	prm->set_float(	p_fLODbias,		0.0f);
	prm->set_bool(	p_bDisplayTargets, false );
	prm->set_float(	p_fElevation,	7.0f );

	//prm->set_float( p_fSunPosAlpha,		2.7f );
	//prm->set_float( p_fSunPosTheta,		0.1f );
	prm->set_float( p_fSunPosAlpha,		1.38f );
	prm->set_float( p_fSunPosTheta,		1.09f );
	prm->set_float( p_fSunShininess,	84.0f );
	prm->set_float( p_fSunStrength,		12.0f );

#ifdef CPU_NORMALS
	prm->set_bool(	p_bSmooth,			false );
#else
	prm->set_bool(	p_bSmooth,			true );
#endif
	prm->set_float(	p_bReflRefrStrength,0.1f );

	prm->set_float( p_fWaterColourR,	0.17f );
	prm->set_float( p_fWaterColourG,	0.27f );
	prm->set_float( p_fWaterColourB,	0.26f );

	prm->set_bool(	p_bAsPoints,		false );
	prm->set_bool(	p_bDrawDuckie,		true );
	prm->set_bool(	p_bDrawIsland,		false );
	prm->set_bool(	p_bDiffuseRefl,		false );

	prm->set_active_parameter(p_fStrength);
}

//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform matrices.
//-----------------------------------------------------------------------------
void Shader_Water::SetupMatrices()
{
	D3DXMATRIXA16 matWorld,matProj;
	D3DXMatrixIdentity(&matWorld);
	g_pd3dDevice->SetTransform( D3DTS_WORLD, &matWorld );
	g_pd3dDevice->SetTransform( D3DTS_VIEW, &(camera_mouse->view) );
//	g_pd3dDevice->SetTransform( D3DTS_PROJECTION, &(camera_mouse->proj) );
}

float Shader_Water::getheight(float x, float z)
{
	//	return displacement*0.5*(1+sin(0.15*x)*sin(z));
	return 0;
}

void Shader_Water::set_clipplane_height(float height)
{
	float plane[4];

	g_pd3dDevice->GetClipPlane(0,plane);

	plane[3] = height;

	g_pd3dDevice->SetClipPlane(0,plane);
}

//  ..  ׷
// ޺,  ü, ׷͵ ..
//  ⿡ ٴ ؽİ .. Ф
void Shader_Water::render_scene(int mode)
{
	// if displaced==true, objects should be displaced according to the waveheight at their location, so the reflections will match

	// set lightning
/*	sun.Direction.x = -cos(prm->get_float(p_fSunPosTheta))*sin(prm->get_float(p_fSunPosAlpha));
	sun.Direction.y = -sin(prm->get_float(p_fSunPosTheta));
	sun.Direction.z = -cos(prm->get_float(p_fSunPosTheta))*cos(prm->get_float(p_fSunPosAlpha));
	if(mode==rsm_reflection)
	{
		//sun.Direction.x = -sun.Direction.x;
		sun.Direction.y = -sun.Direction.y;
		//sun.Direction.z = -sun.Direction.z;
	}
	sun.Diffuse.r = 2.0f;
	sun.Diffuse.g = 2.0f;
	sun.Diffuse.b = 2.0f;
	sun.Diffuse.a = 1.0f;
	sun.Ambient.a = 1.0f;
	sun.Ambient.r = 0.2f;
	sun.Ambient.g = 0.3f;
	sun.Ambient.b = 0.3f;
	sun.Specular.r = 1.0f;
	sun.Specular.g = 1.0f;
	sun.Specular.b = 1.0f;
	sun.Specular.a = 1.0f;

	sun.Attenuation0 = 1.0f;
	sun.Type = D3DLIGHT_DIRECTIONAL;
	g_pd3dDevice->SetLight(0, &sun);
	g_pd3dDevice->LightEnable( 0, true);		

	sun.Diffuse.r = 1.0f;
	sun.Diffuse.g = 1.0f;
	sun.Diffuse.b = 1.0f;
	g_pd3dDevice->SetLight(0, &sun);
*/
//	if (prm->get_bool(p_bDrawIsland)) 
	{	
		if (mode == rsm_refraction)
		{
			//g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW  );	//lalal
			
	/*		LPDIRECT3DVERTEXBUFFER9 vb;
			LPDIRECT3DINDEXBUFFER9 ib;
			island->GetVertexBuffer(&vb);
			island->GetIndexBuffer(&ib);	
			g_pd3dDevice->SetStreamSource(0, vb, 0, island->GetNumBytesPerVertex());
			g_pd3dDevice->SetIndices(ib);
			g_pd3dDevice->SetFVF( island->GetFVF() );
*/
			terrain_effect->Begin(NULL,NULL);
			terrain_effect->BeginPass(0);			
			terrain_effect->SetMatrix("mViewProj",&(camera_mouse->viewproj));
			terrain_effect->SetMatrix("mView",&(camera_mouse->view));			
			float sa = prm->params[p_fSunPosAlpha].fData, st = prm->params[p_fSunPosTheta].fData;
			terrain_effect->SetVector("sun_vec",&D3DXVECTOR4(cos(st)*sin(sa), sin(st), cos(st)*cos(sa),0));
			terrain_effect->SetFloat("sun_shininess", prm->params[p_fSunShininess].fData);			
			terrain_effect->SetFloat("sun_strength", prm->params[p_fSunStrength].fData);
			terrain_effect->SetVector("watercolour", &D3DXVECTOR4(prm->params[p_fWaterColourR].fData,prm->params[p_fWaterColourG].fData,prm->params[p_fWaterColourB].fData,1));
			terrain_effect->SetVector("view_position", &D3DXVECTOR4(camera_mouse->position.x,camera_mouse->position.y,camera_mouse->position.z,1));
			terrain_effect->SetTexture("texDiffuse",terrain_texture);						
		
//			g_pd3dDevice->DrawIndexedPrimitive(	D3DPT_TRIANGLELIST, 0,	0, island->GetNumVertices(), 0, island->GetNumFaces() );				
			terrain_effect->End();
		} 
		else 
		{
			//g_pd3dDevice->SetMaterial((D3DMATERIAL9*)duck_Materials->GetBufferPointer());
			g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, true);

			g_pd3dDevice->SetRenderState( D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_MATERIAL );
			g_pd3dDevice->SetRenderState( D3DRS_SPECULARMATERIALSOURCE, D3DMCS_MATERIAL );
			g_pd3dDevice->SetRenderState( D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_MATERIAL );
			g_pd3dDevice->SetTexture(0, terrain_texture);

			//AttribTable
			set_clipplane_height(0);
	/*		for(int i=0; i<7; i++)
			{
				LPD3DXMATERIAL mat = (LPD3DXMATERIAL) island_Materials->GetBufferPointer();			
				g_pd3dDevice->SetMaterial(&(mat->MatD3D));
				island->DrawSubset(i);
			}					
	*/	}
	}	

	// restore stuff
	g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, false);		
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW  );
}

//   ϴ°
void Shader_Water::render_refracted_scene()
{
	// set rendertarget
	LPDIRECT3DSURFACE9 target,bb;
	g_pd3dDevice->GetRenderTarget(0, &bb );
	sea->surf_refraction->GetSurfaceLevel( 0,&target );
	g_pd3dDevice->SetRenderTarget(0, target);

	/*g_pd3dDevice->Clear( 0, NULL,D3DCLEAR_ZBUFFER, 0, 1.0f, 0 );
	sea->render_cutter();
	g_pd3dDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_GREATEREQUAL  );	*/

	g_pd3dDevice->Clear( 0, NULL,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB((int)(255*prm->params[p_fWaterColourR].fData),(int)(255*prm->params[p_fWaterColourG].fData),(int)(255*prm->params[p_fWaterColourB].fData)), 1.0f, 0 );
	D3DXMATRIXA16 store, scale;	
	// squach the scene
	g_pd3dDevice->GetTransform(D3DTS_WORLD,&store);
	D3DXMatrixScaling( &scale, 1, 0.75, 1 );
	g_pd3dDevice->MultiplyTransform(D3DTS_WORLD, &scale );
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW  );	
	// add a clip-plane as well	
	float plane[4];

	plane[0] = 0;
	plane[1] = -1;
	plane[2] = 0;
	plane[3] = 1.7*prm->params[p_fStrength].fData;	// a slight offset to avoid seams

	g_pd3dDevice->SetClipPlane(0,plane);
	//g_pd3dDevice->SetRenderState( D3DRS_CLIPPLANEENABLE, 1);

	render_scene(rsm_refraction);

	// restore	
	g_pd3dDevice->SetRenderState( D3DRS_ZFUNC, D3DCMP_LESSEQUAL );
	g_pd3dDevice->SetRenderState( D3DRS_CLIPPLANEENABLE, 0);
	g_pd3dDevice->SetTransform(D3DTS_WORLD, &store);	
	g_pd3dDevice->SetRenderTarget(0, bb);
}

//  ǥ ǥϴ°
void Shader_Water::render_actual_scene()
{
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW  );	
	g_pd3dDevice->SetDepthStencilSurface(g_depthstencil);
	render_scene(rsm_normal);
}

// ݻ 
void Shader_Water::render_reflected_scene()
{
	// set rendertarget
	LPDIRECT3DSURFACE9 target,bb;
	g_pd3dDevice->GetRenderTarget(0, &bb );
	sea->surf_reflection->GetSurfaceLevel( 0,&target );
	g_pd3dDevice->SetRenderTarget(0, target);
	g_pd3dDevice->SetDepthStencilSurface( sea->depthstencil );

	// alpha & z must be cleared
	g_pd3dDevice->Clear( 0, NULL,D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, 0, 1.0f, 0 );

	D3DXMATRIXA16 store, scale;	
	// mirror the scene
	g_pd3dDevice->GetTransform(D3DTS_WORLD,&store);
	D3DXMatrixScaling( &scale, 1, -1, 1 );
	g_pd3dDevice->MultiplyTransform(D3DTS_WORLD, &scale );
	g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW  );		

	// add a clip-plane as well	
	float plane[4];

	plane[0] = 0;
	plane[1] = -1;
	plane[2] = 0;
	plane[3] = 0;

	g_pd3dDevice->SetClipPlane(0,plane);
	g_pd3dDevice->SetRenderState( D3DRS_CLIPPLANEENABLE, 1);

	render_scene(rsm_reflection);

	// restore
	g_pd3dDevice->SetRenderState( D3DRS_CLIPPLANEENABLE, 0);
	g_pd3dDevice->SetTransform(D3DTS_WORLD, &store);	
	g_pd3dDevice->SetRenderTarget(0, bb);
	g_pd3dDevice->SetDepthStencilSurface( g_depthstencil );
}

void Shader_Water::Rander(D3DXVECTOR3 eyePosition)
{
	static DWORD it=0;
	static DWORD time_prepare, time_total, time_total_start;
	it++;
	// measure fps
	if (it>63)
	{
		time_total = timeGetTime() - time_total_start;
		time_total_start = timeGetTime();
		it = 0;
	}
	DWORD time_prepare_start = timeGetTime();
// Setup the world, view, and projection matrices (this is used to render the frustum)

	SetupMatrices(); // ⼭ Ʈ ..

	D3DXMATRIXA16 world, view, proj;
	g_pd3dDevice->GetTransform(D3DTS_WORLD , &world);
	D3DXMatrixTranslation( &world, 0.0f, -100.0f, 0.0f);

	g_pd3dDevice->GetTransform(D3DTS_VIEW , &view);
	g_pd3dDevice->GetTransform(D3DTS_PROJECTION , &proj);

    D3DXMATRIXA16 worldViewProj = world * view * proj;


	sea->prepare();				// ٴ ׸ غ
	
	time_prepare = timeGetTime() - time_prepare_start;

	

	// render reflections/refractions into textures
	//#ifdef REFRACTION
	render_refracted_scene();
	//#endif		
	render_reflected_scene();		

	// Begin the scene		

	// set rendering states		
	g_pd3dDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	g_pd3dDevice->SetRenderState(D3DRS_ZWRITEENABLE,true);
	g_pd3dDevice->SetRenderState(D3DRS_ZFUNC,D3DCMP_LESSEQUAL);
	g_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,false);		

	// render sea
	//		if (!prm->get_bool(p_bAsPoints))	sea->render_skybox();
	sea->render(&view, &worldViewProj, eyePosition);	

	render_actual_scene();		

	//D3DRS_BLENDOP
	// Render the camera frustum		
	{
		/*			{	// active_camera_view..  °
		D3DXMATRIXA16 store;
		g_pd3dDevice->GetTransform(D3DTS_WORLD,&store);
		g_pd3dDevice->MultiplyTransform(D3DTS_WORLD, &(cameras[0]->invviewproj));
		g_pd3dDevice->SetStreamSource( 0, g_pVB, 0, sizeof(CUSTOMVERTEX) );
		g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
		g_pd3dDevice->DrawPrimitive( D3DPT_LINELIST, 0, 12 );

		g_pd3dDevice->SetTransform(D3DTS_WORLD, &store);
		}
		*/			// Render the projector frustum		
		{
			D3DXMATRIXA16 store;
			g_pd3dDevice->GetTransform(D3DTS_WORLD,&store);
			g_pd3dDevice->MultiplyTransform(D3DTS_WORLD, &(sea->rendering_camera->invviewproj));
			/*				g_pd3dDevice->SetStreamSource( 0, g_pVBproj, 0, sizeof(CUSTOMVERTEX) );
			g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
			g_pd3dDevice->DrawPrimitive( D3DPT_LINELIST, 0, 12 );
			*/
			g_pd3dDevice->SetTransform(D3DTS_WORLD, &store);
		}
	}

	// render the reference plane
	/*
	g_pd3dDevice->SetRenderState(D3DRS_ALPHABLENDENABLE,true);
	g_pd3dDevice->SetStreamSource( 0, g_referencePlaneVB, 0, sizeof(CUSTOMVERTEX) );
	g_pd3dDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLEFAN, 0, 2 );
	*/			
}